package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.Enums.KuaishouHourEnum;
import cn.com.ctop.kuaishou.modules.batch.Enums.KuaishouMinSecEnum;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouSupplyChainService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

@Slf4j
@Component
public class KuaishouOrderLIstGetJob {

    @Autowired
    private IKuaishouSupplyChainService ikuaishouSupplyChainService;

    /**
     * 昨日快分销数据获取
     **/
    @XxlJob("yesterdayOrderGet")
    public void yesterdayOrderGet() throws Exception {
        log.info("-------------------------获取快分销订单昨日数据开始数据开始--------------------------");
        String cookie = ikuaishouSupplyChainService.getCookie();
        if (Check.isNull(cookie)) {
            log.error("cookie数据为空");
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        JSONArray hourList = KuaishouHourEnum.getHourList();
        JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
        for (int i = 0; i < hourList.size(); i++) {
            String hour = hourList.getString(i);
            for (int j = 0; j < minSecList.size(); j++) {
                JSONObject minSecJson = minSecList.getJSONObject(j);
                String startTime = minSecJson.getString("startTime");
                String endTime = minSecJson.getString("endTime");
                String start = date + hour + startTime;
                String end = date + hour + endTime;
                System.err.println("开始时间：" + start);
                System.err.println("结束时间：" + end);
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Long orderCreateTimeStart = sdf.parse(start).getTime();
                Long orderCreateTimeEnd = sdf.parse(end).getTime();
                ikuaishouSupplyChainService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);

            }
        }
    }


    /**
     * 当前小时数据获取
     **/
    @XxlJob("nowHourOrderGet")
    public void nowHourOrderGet() throws Exception {
        log.info("-------------------------获取快分销订单当天当小时数据开始--------------------------");
        String cookie = ikuaishouSupplyChainService.getCookie();
        if (Check.isNull(cookie)) {
            log.error("cookie数据为空");
            return;
        }
        String date = DateUtils.getNowDate("yyyy-MM-dd");
        Integer nowHour = DateUtils.getNowHour();
        JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
        for (int j = 0; j < minSecList.size(); j++) {
            JSONObject minSecJson = minSecList.getJSONObject(j);
            String startTime = minSecJson.getString("startTime");
            String endTime = minSecJson.getString("endTime");
            String start = date + " " + nowHour + ":" + startTime;
            String end = date + " " + nowHour + ":" + endTime;
            System.err.println("开始时间：" + start);
            System.err.println("结束时间：" + end);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Long orderCreateTimeStart = sdf.parse(start).getTime();
            Long orderCreateTimeEnd = sdf.parse(end).getTime();
            ikuaishouSupplyChainService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);
        }
    }


    /**
     * 当天前前小时数据获取
     **/
    @XxlJob("frontHourOrderGet")
    public void frontHourOrderGet() throws Exception {
        log.info("-------------------------获取快分销订单当天前一小时数据开始--------------------------");
        String cookie = ikuaishouSupplyChainService.getCookie();
        if (Check.isNull(cookie)) {
            log.error("cookie数据为空");
            return;
        }
        String date = DateUtils.getNowDate("yyyy-MM-dd");
        Integer nowHour = DateUtils.getNowHour();
        Integer hour = null;
        if (nowHour == 0) {
            date = DateUtils.getAnotherDay("yyyy-MM-dd", date, -1);
            hour = 23;
        } else {
            hour = nowHour - 1;
        }
        JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
        for (int j = 0; j < minSecList.size(); j++) {
            JSONObject minSecJson = minSecList.getJSONObject(j);
            String startTime = minSecJson.getString("startTime");
            String endTime = minSecJson.getString("endTime");
            String start = date + " " + hour + ":" + startTime;
            String end = date + " " + hour + ":" + endTime;
            System.err.println("开始时间：" + start);
            System.err.println("结束时间：" + end);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Long orderCreateTimeStart = sdf.parse(start).getTime();
            Long orderCreateTimeEnd = sdf.parse(end).getTime();
            ikuaishouSupplyChainService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);
        }
    }


    public static void main(String[] args) throws ParseException {
        String date = DateUtils.getNowDate("yyyy-MM-dd");
        Integer nowHour = 0;
        Integer hour = null;
        if (nowHour == 0) {
            date = DateUtils.getAnotherDay("yyyy-MM-dd", date, -1);
            hour = 23;
        } else {
            hour = nowHour - 1;
        }

        System.err.println(date);
        System.err.println(hour);


    }
}
