package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaishouProjectVideoGet;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouVideoGetService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouProjectVideoGetService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 快手-项目下视频信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-05-21
 */
@Slf4j
@Api(tags = "快手-项目下视频信息")
@RestController
@RequestMapping("/kuaishouProjectVideoGet")
public class KuaishouProjectVideoGetController {
    @Autowired
    private IKuaishouProjectVideoGetService kuaishouProjectVideoGetService;
    @Autowired
    private IKuaiShouVideoGetService videoGetService;

    /**
     * 分页列表查询
     *
     * @param kuaishouProjectVideoGet
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/list")
    public Result<Object> queryPageList(KuaishouProjectVideoGet kuaishouProjectVideoGet,
                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            return kuaishouProjectVideoGetService.queryPageList(kuaishouProjectVideoGet, pageNo, pageSize);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 素材关联被拒创意
     */
    @GetMapping(value = "/queryCreativeByVideo")
    public Result<Object> queryCreativeByVideo(String signature, Long projectId, Long accountId, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo, @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(signature) || Check.isNull(projectId)) {
                return Result.error("缺失参数");
            }
            List<String> photoIds = videoGetService.queryPhotoIdsByProjectAndMd5(projectId, signature, null);
            if (!Check.isNull(photoIds)) {
                return kuaishouProjectVideoGetService.queryCreativeBySignature(photoIds, accountId, pageNo, pageSize);
            }
            return Result.ok("没有查询到素材");
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("查询失败");
        }
    }

    /**
     * 素材数量统计
     *
     * @return
     */
    @GetMapping(value = "/getVideoTotalNumber")
    public Result<Object> getVideoTotalNumber(Long projectId) {
        try {
            if (Check.isNull(projectId)) {
                return Result.error("请选择项目");
            }
            return kuaishouProjectVideoGetService.getVideoTotalNumber(projectId);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 下架素材
     *
     * @return
     */
    @PostMapping(value = "/offShelfVideo")
    public Result<Object> offShelfVideo(@RequestBody JSONObject data) {
        try {
            if (Check.isNull(data.getString("id"))) {
                return Result.error("缺失参数");
            }
            KuaishouProjectVideoGet videoGet = new KuaishouProjectVideoGet();
            videoGet.setStatus(1);
            videoGet.setId(data.getString("id"));
            kuaishouProjectVideoGetService.updateById(videoGet);
            return Result.ok("success");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("下架失败");
    }

    /**
     * 下架素材
     *
     * @return
     */
    @PostMapping(value = "/offShelfVideos")
    public Result<Object> offShelfVideos(@RequestBody JSONObject data) {
        try {
            Long projectId = data.getLong("projectId");
            if (Check.isNull(projectId)) {
                return Result.error("请选择项目");
            }
            JSONArray accountId = data.getJSONArray("accountId");
            if (Check.isNull(accountId)) {
                return Result.error("请选择账户");
            }
            String signature = data.getString("signature");
            if (Check.isNull(signature)) {
                return Result.error("请选择下架的视频");
            }
            KuaishouProjectVideoGet videoGet = new KuaishouProjectVideoGet();
            QueryWrapper<KuaishouProjectVideoGet> queryWrapper = new QueryWrapper<>();
            videoGet.setStatus(1);
            queryWrapper.eq("project_id", projectId);
            queryWrapper.eq("signature", signature);
            kuaishouProjectVideoGetService.update(videoGet, queryWrapper);
            List<String> photoIds = videoGetService.queryPhotoIdsByProjectAndMd5(projectId, signature, accountId);
            for (Object photoId : photoIds) {
                kuaishouProjectVideoGetService.deleteCreativesByPhotoId(photoId);
            }
            return Result.ok("success");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("下架失败");
    }

    /**
     * 素材详情数据
     * put_start_time
     *
     * @return
     */
    @GetMapping(value = "/getVideoDetails")
    public Result<Object> getVideoDetails(String signature, String id) {
        try {
            if (Check.isNull(signature) || Check.isNull(id)) {
                return Result.error("缺失参数");
            }
            return kuaishouProjectVideoGetService.getVideoDetails(signature, id);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 素材已关联账户数据
     * put_start_time
     *
     * @return
     */
    @GetMapping(value = "/getAccountVideos")
    public Result<Object> getAccountVideos(String signature, Long projectId, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo, @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(signature)) {
                return Result.error("请选择素材");
            }
            if (Check.isNull(projectId)) {
                return Result.error("请选择项目");
            }
            return kuaishouProjectVideoGetService.getAccountVideos(signature, projectId, pageNo, pageSize);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 添加
     *
     * @param kuaishouProjectVideoGet
     * @return
     */
    @AutoLog(value = "快手-项目下视频信息-添加")
    @ApiOperation(value = "快手-项目下视频信息-添加", notes = "快手-项目下视频信息-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouProjectVideoGet> add(@RequestBody KuaishouProjectVideoGet kuaishouProjectVideoGet) {
        Result<KuaishouProjectVideoGet> result = new Result<>();
        try {
            kuaishouProjectVideoGetService.save(kuaishouProjectVideoGet);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    @GetMapping(value = "/clearProjectVideo")
    public Result<Object> clearProjectVideo(Long projectId) {
        try {
            kuaishouProjectVideoGetService.clearProjectVideo(projectId,1,null);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return Result.ok("执行完成");
    }

    /**
     * 编辑
     *
     * @param kuaishouProjectVideoGet
     * @return
     */
    @AutoLog(value = "快手-项目下视频信息-编辑")
    @ApiOperation(value = "快手-项目下视频信息-编辑", notes = "快手-项目下视频信息-编辑")
    @PostMapping(value = "/edit")
    public Result<KuaishouProjectVideoGet> edit(@RequestBody KuaishouProjectVideoGet kuaishouProjectVideoGet) {
        Result<KuaishouProjectVideoGet> result = new Result<KuaishouProjectVideoGet>();
        KuaishouProjectVideoGet kuaishouProjectVideoGetEntity = kuaishouProjectVideoGetService.getById(kuaishouProjectVideoGet.getId());
        if (kuaishouProjectVideoGetEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouProjectVideoGetService.updateById(kuaishouProjectVideoGet);
            if (ok) {
                result.success("操作成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-项目下视频信息-通过id删除")
    @ApiOperation(value = "快手-项目下视频信息-通过id删除", notes = "快手-项目下视频信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouProjectVideoGetService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-项目下视频信息-批量删除")
    @ApiOperation(value = "快手-项目下视频信息-批量删除", notes = "快手-项目下视频信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouProjectVideoGet> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouProjectVideoGet> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouProjectVideoGetService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-项目下视频信息-通过id查询")
    @ApiOperation(value = "快手-项目下视频信息-通过id查询", notes = "快手-项目下视频信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouProjectVideoGet> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouProjectVideoGet> result = new Result<>();
        KuaishouProjectVideoGet kuaishouProjectVideoGet = kuaishouProjectVideoGetService.getById(id);
        if (kuaishouProjectVideoGet == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouProjectVideoGet);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouProjectVideoGet> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouProjectVideoGet kuaishouProjectVideoGet = JSON.parseObject(deString, KuaishouProjectVideoGet.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouProjectVideoGet, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouProjectVideoGet> pageList = kuaishouProjectVideoGetService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "快手-项目下视频信息列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouProjectVideoGet.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手-项目下视频信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouProjectVideoGet> listKuaishouProjectVideoGets = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouProjectVideoGet.class, params);
                kuaishouProjectVideoGetService.saveBatch(listKuaishouProjectVideoGets);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouProjectVideoGets.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
